@@ -41,7 +41,6 @@ namespace Beagle.Filters {
 	public class FilterOpenOffice : Beagle.Daemon.Filter {
 
 		private Hashtable hotStyles;
-		private bool odtFormat;
 
 		private Hashtable attr_to_index = null;
 		private Hashtable InterestingAttribute {
@@ -59,6 +58,11 @@ namespace Beagle.Filters {
 		}
 
 		public FilterOpenOffice () 
+		{
+			SnippetMode = true;
+		}
+
+		protected override void RegisterSupportedTypes ()
 		{
 			// OO 1.0 mime types
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.writer"));
@@ -79,9 +83,6 @@ namespace Beagle.Filters {
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.presentation-template"));
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.graphics"));
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.graphics-template"));
-
-			SnippetMode = true;
-			odtFormat = false;
 		}
 		
 		// Parse the "style" nodes and mark appropriate styles as *HOT*
@@ -177,7 +178,7 @@ namespace Beagle.Filters {
 		}
 
 		private Stack hot_nodes = new Stack ();
-		private string strPartText = "";
+		private string strPartText = String.Empty;
 		private bool bPartHotStyle = false;
 		private Stack hot_container_nodes = new Stack ();
 
@@ -218,8 +219,8 @@ namespace Beagle.Filters {
 			} else {
 				strTemp = strPartText + paramStr;
 				strPartText = strTemp;
-				paramStr = "";
-				strTemp = "";
+				paramStr = String.Empty;
+				strTemp = String.Empty;
 			}
 					
 			// Enable *HOT* just before appending the text
@@ -232,8 +233,7 @@ namespace Beagle.Filters {
 			} else 
 				bPartHotStyle |= false;
 
-			if (paramStr.Length > 0)
-				AppendText (paramStr);
+			AppendText (paramStr);
 
 			if (strPartText.Trim().Length < 1)
 				bPartHotStyle = false;
@@ -256,7 +256,7 @@ namespace Beagle.Filters {
 				AppendText (strPartText);
 				if (IsHot)
 					HotDown ();
-				strPartText = "";
+				strPartText = String.Empty;
 			}
 			bPartHotStyle = false;
 
@@ -421,15 +421,14 @@ namespace Beagle.Filters {
 				}
 				reader.Read ();
 			}
-			if (slideCount != null)
-				AddProperty (Beagle.Property.NewUnsearched ("fixme:slide-count", slideCount));
+
+			AddProperty (Beagle.Property.NewUnsearched ("fixme:slide-count", slideCount));
 		}
 
 		private void ExtractMetadata (XmlReader reader)
 		{
-			do {
-				reader.Read ();
-			} while (reader.Depth < 2);
+			while (reader.Read () && reader.Depth < 2)
+				;
 
 			while (reader.Depth >= 2) {
 				if (reader.Depth != 2 || reader.NodeType != XmlNodeType.Element) {
@@ -459,11 +458,9 @@ namespace Beagle.Filters {
 					
 				case "meta:document-statistic":
 					string attr = reader.GetAttribute ("meta:page-count");
-					if (attr != null)
-						AddProperty (Beagle.Property.NewUnsearched ("fixme:page-count", attr));
+					AddProperty (Beagle.Property.NewUnsearched ("fixme:page-count", attr));
 					attr = reader.GetAttribute ("meta:word-count");
-					if (attr != null)
-						AddProperty (Beagle.Property.NewUnsearched ("fixme:word-count", attr));
+					AddProperty (Beagle.Property.NewUnsearched ("fixme:word-count", attr));
 
 					// Both writer and calc uses this attribute.  writer stores the
 					// count of tables in a sxw whereas calc stores the count of
@@ -478,16 +475,14 @@ namespace Beagle.Filters {
 					string name = reader.GetAttribute ("meta:name");
 					reader.Read ();
 
-					if (reader.Value != "") {
-						AddProperty (Beagle.Property.New ("fixme:UserDefined-" + name,
+					AddProperty (Beagle.Property.New ("fixme:UserDefined-" + name,
 										  reader.Value));
-					}
 					break;
 
 				case "meta:keyword":
 					reader.Read ();
 					AddProperty (Beagle.Property.New ("fixme:keywords",
-									  +  reader.Value));
+									  reader.Value));
 					break;
 				}
 				
@@ -503,8 +498,8 @@ namespace Beagle.Filters {
 			try {
 				zip = new ZipFile (info.FullName);
 
-				if (MimeType.StartsWith ("application/vnd.oasis.opendocument."))
-					odtFormat = true;
+				//if (MimeType.StartsWith ("application/vnd.oasis.opendocument."))
+				//	odtFormat = true;
 
 			} catch (Exception) {
 				Logger.Log.Error ("Unable to open {0}.  Probably an invalid OpenOffice document.", 
