@@ -325,7 +325,8 @@ namespace Beagle.Daemon {
 					arg_debug_memory = true;
 					break;
 
-				case "--no-shapshots":
+				case "--no-snapshots":
+				case "--no-snapshot":
 					arg_heap_shot_snapshots = false;
 					break;
 
@@ -494,6 +495,9 @@ namespace Beagle.Daemon {
 				Logger.Log.Debug ("Unable to establish a connection to the X server");
 			XSetIOErrorHandler (BeagleXIOErrorHandler);
 
+			// Lower our CPU priority
+			SystemPriorities.Renice (7);
+
 			QueryDriver.Init ();
 			Server.Init ();
 
