@@ -1,6 +1,7 @@
 //
 // BuildIndex.cs
 //
+// Copyright (C) 2007 Debajyoti Bera <dbera.web@gmail.com>
 // Copyright (C) 2005 Novell, Inc.
 //
 
@@ -48,12 +49,10 @@ namespace Beagle.Daemon
 {
 	class BuildIndex 
 	{
-		static string [] argv;
-
-		static bool arg_recursive = false, arg_delete = false, arg_debug = false, arg_cache_text = false, arg_disable_filtering = false, arg_disable_restart = false, arg_disable_directories = false;
-
-		static Hashtable remap_table = new Hashtable ();
-
+		static bool arg_recursive, arg_delete, arg_debug;
+		static bool arg_cache_text, arg_disable_filtering;
+		static bool arg_disable_restart, arg_disable_directories;
+		static bool arg_disable_on_battery;
 		static string arg_output, arg_tag, arg_source;
 
 		/////////////////////////////////////////////////////////
@@ -81,7 +80,7 @@ namespace Beagle.Daemon
 
 		static LuceneIndexingDriver driver;
 
-		static bool crawling = true, indexing = true, shutdown = false, restart = false;
+		static bool indexing = true, restart = false;
 
 		static ArrayList allowed_patterns = new ArrayList ();
 		static ArrayList denied_patterns = new ArrayList ();
@@ -146,23 +145,6 @@ namespace Beagle.Daemon
 					arg_cache_text = true;
 					break;
 
-				/*
-				case "--remap":
-					if (next_arg == null) 
-						break;
-					
-					int j = next_arg.IndexOf (":");
-
-					if (j == -1) {
-						Logger.Log.Error ("Invalid remap argument: {0}", next_arg);
-						Environment.Exit (1);
-					}
-					
-					remap_table [next_arg.Substring (0, j)] = next_arg.Substring (j+1);
-
-					++i;
-					break;
-				*/
 				case "--target":
 					if (next_arg != null)
 						arg_output = Path.IsPathRooted (next_arg) ? next_arg : Path.GetFullPath (next_arg);
@@ -173,6 +155,10 @@ namespace Beagle.Daemon
 					arg_disable_filtering = true;
 					break;
 
+				case "--disable-on-battery":
+					arg_disable_on_battery = true;
+					break;
+
 				case "--allow-pattern":
 					if (next_arg == null)
 						break;
@@ -228,8 +214,6 @@ namespace Beagle.Daemon
 				}
 			}
 			
-			argv = args;
-			
 			/////////////////////////////////////////////////////////
 				
 			if (arg_output == null) {
@@ -276,6 +260,13 @@ namespace Beagle.Daemon
 				}
 			}
 
+			if (SystemInformation.UsingBattery && arg_disable_on_battery) {
+				Log.Always ("Indexer is disabled when on battery power (--disable-on-battery)");
+				Environment.Exit (0);
+			}
+
+			Log.Always ("Starting beagle-build-index (pid {0}) at {1}", Process.GetCurrentProcess ().Id, DateTime.Now);
+
 			// Set system priorities so we don't slow down the system
 			SystemPriorities.ReduceIoPriority ();
 			SystemPriorities.SetSchedulerPolicyBatch ();
@@ -292,25 +283,20 @@ namespace Beagle.Daemon
 			// Set up signal handlers
 			SetupSignalHandlers ();
 
-			Thread crawl_thread, index_thread, monitor_thread = null;
+			Thread monitor_thread = null;
 
 			Stopwatch watch = new Stopwatch ();
 			watch.Start ();
 
-			// Start the thread that does the crawling
-			crawl_thread = ExceptionHandlingThread.Start (new ThreadStart (CrawlWorker));
-
-			// Start the thread that does the actual indexing
-			index_thread = ExceptionHandlingThread.Start (new ThreadStart (IndexWorker));
-
 			if (!arg_disable_restart) {
 				// Start the thread that monitors memory usage.
 				monitor_thread = ExceptionHandlingThread.Start (new ThreadStart (MemoryMonitorWorker));
 			}
 
-			// Join all the threads so that we know that we're the only thread still running
-			crawl_thread.Join ();
-			index_thread.Join ();
+			// Start indexworker to do the crawling and indexing
+			IndexWorker ();
+
+			// Join any threads so that we know that we're the only thread still running
 			if (monitor_thread != null)
 				monitor_thread.Join ();
 
@@ -318,7 +304,7 @@ namespace Beagle.Daemon
 			Logger.Log.Debug ("Elapsed time {0}.", watch);
 
 			if (restart) {
-				Logger.Log.Debug ("Restarting helper");
+				Logger.Log.Debug ("Restarting beagle-build-index");
 				Process p = new Process ();
 				p.StartInfo.UseShellExecute = false;
 				// FIXME: Maybe this isn't the right way to do things?  It should be ok,
@@ -327,58 +313,120 @@ namespace Beagle.Daemon
 				p.StartInfo.Arguments = String.Join (" ", Environment.GetCommandLineArgs ());
 				p.Start ();
 			}
+
+			Log.Always ("Exiting beagle-build-index (pid {0}) at {1}", Process.GetCurrentProcess ().Id, DateTime.Now);
 		}
 		
 		/////////////////////////////////////////////////////////////////
 		
-		static void CrawlWorker ()
+		static void IndexWorker ()
 		{
-			Logger.Log.Debug ("Starting CrawlWorker");
-			
+			Log.Debug ("Starting IndexWorker");
+
 			try {
-				int count_dirs = 0;
-				int count_files = 0;
-			
-				while (pending_directories.Count > 0) {
-					DirectoryInfo dir = (DirectoryInfo) pending_directories.Dequeue ();
-
-					if (! arg_disable_directories)
-						pending_files.Enqueue (dir);
-
-					try {
-						if (arg_recursive)
-							foreach (DirectoryInfo subdir in DirectoryWalker.GetDirectoryInfos (dir))
-								if (!Ignore (subdir)
-								    && !FileSystem.IsSpecialFile (subdir.FullName))
-									pending_directories.Enqueue (subdir);
-					
-						foreach (FileInfo file in DirectoryWalker.GetFileInfos (dir))
-							if (!Ignore (file)
-							    && !FileSystem.IsSpecialFile (file.FullName)) {
-								pending_files.Enqueue (file);
-								count_files ++;
-							}
-					
-					} catch (DirectoryNotFoundException e) {}
-				
-					if (shutdown)
-						break;
-				
-					count_dirs++;
-				}
-
-				Logger.Log.Debug ("Scanned {0} files and directories in {1} directories", count_dirs + count_files, count_dirs);
-			} finally {
-				Logger.Log.Debug ("CrawlWorker Done");
-
-				crawling = false;
+				DoIndexing ();
+			} catch (Exception e) {
+				Log.Debug ("Encountered exception while indexing: {0}", e);
 			}
+
+			Logger.Log.Debug ("IndexWorker Done");
+			indexing = false;
+		}
+
+		static void DoIndexing ()
+		{
+			int count_dirs = 0;
+			int count_files = 0;
+
+			Indexable indexable;
+			IndexerRequest pending_request;
+			pending_request = new IndexerRequest ();
+			Queue modified_directories = new Queue ();
+			
+			while (pending_directories.Count > 0) {
+				DirectoryInfo dir = (DirectoryInfo) pending_directories.Dequeue ();
+
+				if (! arg_disable_directories)
+					AddToRequest (pending_request, DirectoryToIndexable (dir, modified_directories));
+
+				try {
+					if (arg_recursive)
+						foreach (DirectoryInfo subdir in DirectoryWalker.GetDirectoryInfos (dir))
+							if (!Ignore (subdir)
+							    && !FileSystem.IsSpecialFile (subdir.FullName))
+								pending_directories.Enqueue (subdir);
+				
+					foreach (FileInfo file in DirectoryWalker.GetFileInfos (dir))
+						if (!Ignore (file)
+						    && !FileSystem.IsSpecialFile (file.FullName)) {
+							AddToRequest (pending_request, FileToIndexable (file));
+							count_files ++;
+						}
+				
+				} catch (DirectoryNotFoundException) {}
+			
+				if (Shutdown.ShutdownRequested)
+					break;
+			
+				count_dirs++;
+			}
+
+			Logger.Log.Debug ("Scanned {0} files and directories in {1} directories", count_dirs + count_files, count_dirs);
+
+			if (Shutdown.ShutdownRequested) {
+				backing_fa_store.Flush ();
+				return;
+			}
+
+			// Time to remove deleted directories from the index and attributes store
+			while (modified_directories.Count > 0) {
+				DirectoryInfo subdir = (DirectoryInfo) modified_directories.Dequeue ();
+				Logger.Log.Debug ("Checking {0} for deleted files and directories", subdir.FullName);
+
+				// Get a list of all documents from lucene index with ParentDirUriPropKey set as that of subdir
+				ICollection all_dirent = GetAllItemsInDirectory (subdir);
+				foreach (Dirent info in all_dirent) {
+					// check if the item exists
+					if (File.Exists (info.FullName) || 
+					    (info.IsDirectory && Directory.Exists (info.FullName)))
+						continue;
+
+					if (info.IsDirectory)
+						// Recursively remove deleted subdirectories
+						modified_directories.Enqueue (new DirectoryInfo (info.FullName));
+					
+					// remove
+					Uri uri = UriFu.PathToFileUri (info.FullName);
+					indexable = new Indexable (IndexableType.Remove, uri);
+					AddToRequest (pending_request, indexable);
+				}
+			}
+			
+			// Call Flush until our request is empty.  We have to do this in a loop
+			// because children can get added back to the pending request in a flush.
+			while (pending_request.Count > 0) {
+				if (Shutdown.ShutdownRequested)
+					break;
+
+				FlushIndexer (driver, pending_request);
+			}
+
+			backing_fa_store.Flush ();
+
+			if (Shutdown.ShutdownRequested)
+				return;
+
+			Logger.Log.Debug ("Optimizing index");
+			driver.OptimizeNow ();
 		}
 		
 		/////////////////////////////////////////////////////////////////
 
 		static void AddToRequest (IndexerRequest request, Indexable indexable)
 		{
+			if (indexable == null)
+				return;
+
 			// Disable filtering and only index file attributes
 			if (arg_disable_filtering)
 				indexable.Filtering = IndexableFiltering.Never;
@@ -395,6 +443,12 @@ namespace Beagle.Daemon
 			indexable.Source = arg_source;
 
 			request.Add (indexable);
+
+			if (! Shutdown.ShutdownRequested && request.Count >= BATCH_SIZE) {
+				Logger.Log.Debug ("Flushing driver, {0} items in queue", request.Count);
+				FlushIndexer (driver, request);
+				// FlushIndexer clears the pending_request
+			}
 		}
 
 		static IndexerReceipt [] FlushIndexer (IIndexer indexer, IndexerRequest request)
@@ -402,6 +456,10 @@ namespace Beagle.Daemon
 			IndexerReceipt [] receipts;
 			receipts = indexer.Flush (request);
 
+			// Flush will return null if it encounters a shutdown during flushing
+			if (receipts == null)
+				return null;
+
 			ArrayList pending_children;
 			pending_children = new ArrayList ();
 
@@ -447,6 +505,10 @@ namespace Beagle.Daemon
 			}
 
 			request.Clear (); // clear out the old request
+
+			if (Shutdown.ShutdownRequested)
+				return receipts;
+
 			foreach (Indexable i in pending_children) // and then add the children
 				AddToRequest (request, i);
 			
@@ -455,7 +517,7 @@ namespace Beagle.Daemon
 
 		static Indexable FileToIndexable (FileInfo file)
 		{
-			if (!file.Exists || Ignore (file) || fa_store.IsUpToDate (file.FullName))
+			if (!file.Exists || Ignore (file) || fa_store.IsUpToDateAndFiltered (file.FullName))
 				return null;
 
 			// Create the indexable and add the standard properties we
@@ -463,6 +525,7 @@ namespace Beagle.Daemon
 			Uri uri = UriFu.PathToFileUri (file.FullName);
 			Indexable indexable = new Indexable (uri);
 			indexable.Timestamp = file.LastWriteTimeUtc;
+			indexable.Crawled = true;
 			FSQ.AddStandardPropertiesToIndexable (indexable, file.Name, Guid.Empty, false);
 
 			// Store directory name in the index
@@ -506,84 +569,6 @@ namespace Beagle.Daemon
 			return indexable;
 		}
 		
-		static void IndexWorker ()
-		{
-			Logger.Log.Debug ("Starting IndexWorker");
-			Queue modified_directories = new Queue ();
-			
-			try {
-				Indexable indexable;
-				IndexerRequest pending_request;
-				pending_request = new IndexerRequest ();
-			
-				while (!shutdown) {
-					if (pending_files.Count > 0) {
-						Object file_or_dir_info = pending_files.Dequeue ();
-
-						if (file_or_dir_info is DirectoryInfo)
-							indexable = DirectoryToIndexable ((DirectoryInfo) file_or_dir_info, modified_directories);
-						else
-							indexable = FileToIndexable ((FileInfo) file_or_dir_info);
-							
-						if (indexable == null)
-							continue;
-					
-						AddToRequest (pending_request, indexable);
-					
-						if (pending_request.Count >= BATCH_SIZE) {
-							Logger.Log.Debug ("Flushing driver, {0} items in queue", pending_request.Count);
-							FlushIndexer (driver, pending_request);
-							// FlushIndexer clears the pending_request
-						}
-
-					} else if (crawling) {
-						//Logger.Log.Debug ("IndexWorker: La la la...");
-						Thread.Sleep (50);
-					} else {
-						break;
-					}
-				}
-
-				// Time to remove deleted directories from the index and attributes store
-				while (modified_directories.Count > 0) {
-					DirectoryInfo subdir = (DirectoryInfo) modified_directories.Dequeue ();
-					Logger.Log.Debug ("Checking {0} for deleted files and directories", subdir.FullName);
-
-					// Get a list of all documents from lucene index with ParentDirUriPropKey set as that of subdir
-					ICollection all_dirent = GetAllItemsInDirectory (subdir);
-					foreach (Dirent info in all_dirent) {
-						// check if the item exists
-						if (File.Exists (info.FullName) || 
-						    (info.IsDirectory && Directory.Exists (info.FullName)))
-							continue;
-
-						if (info.IsDirectory)
-							// Recursively remove deleted subdirectories
-							modified_directories.Enqueue (new DirectoryInfo (info.FullName));
-						
-						// remove
-						Uri uri = UriFu.PathToFileUri (info.FullName);
-						indexable = new Indexable (IndexableType.Remove, uri);
-						AddToRequest (pending_request, indexable);
-					}
-				}
-				
-				// Call Flush until our request is empty.  We have to do this in a loop
-				// because children can get added back to the pending request in a flush.
-				while (pending_request.Count > 0)
-					FlushIndexer (driver, pending_request);
-
-				backing_fa_store.Flush ();
-
-				Logger.Log.Debug ("Optimizing index");
-				driver.OptimizeNow ();
-			} finally {
-				Logger.Log.Debug ("IndexWorker Done");
-
-				indexing = false;
-			}
-		}
-
 		class Dirent {
 			private bool is_directory;
 			private string path;
@@ -706,7 +691,7 @@ namespace Beagle.Daemon
 			const double threshold = 6.0;
 			int last_vmrss = 0;
 
-			while (! shutdown && (crawling || indexing)) {
+			while (! Shutdown.ShutdownRequested && indexing) {
 
 				// Check resident memory usage
 				int vmrss = SystemInformation.VmRss;
@@ -718,7 +703,7 @@ namespace Beagle.Daemon
 				if (size > threshold) {
 					Logger.Log.Debug ("Process too big, shutting down!");
 					restart = true;
-					shutdown = true;
+					Shutdown.ShutdownRequested = true;
 					return;
 				} else {
 					Thread.Sleep (3000);
@@ -750,7 +735,7 @@ namespace Beagle.Daemon
 				return;
 
 			Logger.Log.Debug ("Shutdown Requested");
-			shutdown = true;
+			Shutdown.ShutdownRequested = true;
 		}
 
 		/////////////////////////////////////////////////////////////////
@@ -792,17 +777,6 @@ namespace Beagle.Daemon
 		
 		/////////////////////////////////////////////////////////
 		
-		static Uri RemapUri (Uri uri)
-		{
-			// FIXME: This is ghetto
-			foreach (DictionaryEntry dict in remap_table) {
-				if (uri.LocalPath.IndexOf ((string) dict.Key) == -1)
-					continue;
-				return new Uri (uri.LocalPath.Replace ((string) dict.Key, (string) dict.Value));
-			}
-			return uri;
-		}
-		
 		static bool Ignore (DirectoryInfo directory)
 		{
 			if (directory.Name.StartsWith ("."))
