@@ -44,6 +44,10 @@ namespace Beagle.Daemon {
 		public const string SELF_CACHE_TAG = "*self*";
 
 		private string text_cache_dir;
+		internal string TextCacheDir {
+			get { return text_cache_dir; }
+		}
+
 		private SqliteConnection connection;
 
 		private enum TransactionState {
@@ -161,7 +165,7 @@ namespace Beagle.Daemon {
 
 		private static string UriToString (Uri uri)
 		{
-			return UriFu.UriToSerializableString (uri).Replace ("'", "''");
+			return UriFu.UriToEscapedString (uri).Replace ("'", "''");
 		}
 
 		private SqliteCommand NewCommand (string format, params object [] args)
@@ -183,6 +187,7 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		// Returns raw path as stored in the db i.e. relative path wrt the text_cache_dir
 		private string LookupPathRawUnlocked (Uri uri, bool create_if_not_found)
 		{
 			SqliteCommand command;
@@ -206,7 +211,7 @@ namespace Beagle.Daemon {
 			if (path == SELF_CACHE_TAG)
 				return SELF_CACHE_TAG;
 
-			return path != null ? Path.Combine (text_cache_dir, path) : null;
+			return path;
 		}
 
 		// Don't do this unless you know what you are doing!  If you
@@ -236,7 +241,7 @@ namespace Beagle.Daemon {
 					}
 					return uri.LocalPath;
 				}
-				return path;
+				return path != null ? Path.Combine (text_cache_dir, path) : null;
 			}
 		}
 		
@@ -272,7 +277,7 @@ namespace Beagle.Daemon {
 			}
 
 			StreamWriter writer;
-			writer = new StreamWriter (stream);
+			writer = new StreamWriter (new BufferedStream (stream));
 			return writer;
 		}
 
@@ -321,7 +326,7 @@ namespace Beagle.Daemon {
 			StreamReader reader = null;
 			try {
 				Stream stream = new GZipInputStream (file_stream);
-				reader = new StreamReader (stream);
+				reader = new StreamReader (new BufferedStream (stream));
 
 				// This throws an exception if the file isn't compressed as follows:
 				// 1.) IOException on older versions of SharpZipLib
@@ -346,7 +351,7 @@ namespace Beagle.Daemon {
 								"DELETE FROM uri_index WHERE uri='{0}' AND filename='{1}'", 
 								UriToString (uri), path);
 					if (path != SELF_CACHE_TAG)
-						File.Delete (path);
+						File.Delete (Path.Combine (text_cache_dir, path));
 				}
 			}
 		}
