@@ -2,7 +2,7 @@
 // TomboyQueryable.cs
 //
 // Copyright (C) 2004 Christopher Orr
-// Copyright (C) 2004 Novell, Inc.
+// Copyright (C) 2004-2006 Novell, Inc.
 //
 
 //
@@ -26,6 +26,7 @@
 //
 
 using System;
+using System.Collections;
 using System.IO;
 using System.Threading;
 
@@ -35,15 +36,16 @@ using Beagle.Util;
 namespace Beagle.Daemon.TomboyQueryable {
 
 	[QueryableFlavor (Name="Tomboy", Domain=QueryDomain.Local, RequireInotify=false)]
-	public class TomboyQueryable : LuceneQueryable {
-
-		private static Logger log = Logger.Get ("TomboyQueryable");
+	public class TomboyQueryable : LuceneFileQueryable, IIndexableGenerator  {
 
 		string tomboy_dir;
+		Hashtable note_text_cache = UriFu.NewHashtable ();
 
 		public TomboyQueryable () : base ("TomboyIndex")
 		{
-			tomboy_dir = Path.Combine (PathFinder.HomeDir, ".tomboy");
+			tomboy_dir = Environment.GetEnvironmentVariable ("TOMBOY_PATH");
+			if (tomboy_dir == null)
+				tomboy_dir = Path.Combine (PathFinder.HomeDir, ".tomboy");
 		}
 
 		public override void Start () 
@@ -78,24 +80,15 @@ namespace Beagle.Daemon.TomboyQueryable {
 				fsw.EnableRaisingEvents = true;
 			}
 
-			// Crawl all of our existing notes to make sure that
-			// everything is up-to-date.
-			log.Info ("Scanning Tomboy notes...");
+			// Start our crawler process
+			Scheduler.Task task;
+			task = NewAddTask (this);
+			task.Tag = "Crawling Tomboy Notes";
+			task.Source = this;
 
-			Stopwatch stopwatch = new Stopwatch ();
-			int count = 0;
-			stopwatch.Start ();
-			DirectoryInfo dir = new DirectoryInfo (tomboy_dir);
+			ThisScheduler.Add (task);
 
-			foreach (FileInfo file in dir.GetFiles ()) {
-				if (file.Extension == ".note") {
-					IndexNote (file, Scheduler.Priority.Delayed);
-					++count;
-				}
-			}
-
-			stopwatch.Stop ();
-			log.Info ("Scanned {0} notes in {1}", count, stopwatch);
+			Logger.Log.Info ("Tomboy backend started");
 		}
 
 		private bool CheckForExistence ()
@@ -146,20 +139,23 @@ namespace Beagle.Daemon.TomboyQueryable {
 		}
 
 		/////////////////////////////////////////////////
-
-		private static Indexable NoteToIndexable (FileInfo file, Note note)
+		
+		private Indexable NoteToIndexable (FileInfo file, Note note)
 		{
 			Indexable indexable = new Indexable (note.Uri);
 
 			indexable.ContentUri = UriFu.PathToFileUri (file.FullName);
 
 			indexable.Timestamp = note.timestamp;
-			indexable.Type = "Note";
-			indexable.MimeType = "text/plain";
+			indexable.HitType = "Note";
+			indexable.Filtering = IndexableFiltering.AlreadyFiltered;
 
-			// Using dc:title for a note's subject is debateable?
 			indexable.AddProperty (Property.New ("dc:title", note.subject));
-			indexable.AddProperty (Property.NewDate ("fixme:modified", note.timestamp));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:application","tomboy"));
+
+			// We remember the note's text so that we can stuff it in
+			// the TextCache later.
+			note_text_cache [note.Uri] = note.text;
 
 			StringReader reader = new StringReader (note.text);
 			indexable.SetTextReader (reader);
@@ -169,7 +165,7 @@ namespace Beagle.Daemon.TomboyQueryable {
 
 		private void IndexNote (FileInfo file, Scheduler.Priority priority)
 		{
-			if (this.FileAttributesStore.IsUpToDate (file.FullName))
+			if (this.IsUpToDate (file.FullName))
 				return;
 
 			// Try and parse a Note from the given path
@@ -186,7 +182,6 @@ namespace Beagle.Daemon.TomboyQueryable {
 			ThisScheduler.Add (task);
 		}
 
-		
 		private void RemoveNote (string file)
 		{
 			Uri uri = Note.BuildNoteUri (file, "tomboy");
@@ -196,6 +191,22 @@ namespace Beagle.Daemon.TomboyQueryable {
 			ThisScheduler.Add (task);
 		}
 
+		override protected void PostAddHook (Indexable indexable, IndexerAddedReceipt receipt)
+		{
+			base.PostAddHook (indexable, receipt);
+			
+			// Store the note's text in the text cache.
+			// By doing this in the PostAddHook, we ensure that
+			// the TextCache is not modified until we are
+			// sure that the note was actually indexed.
+			string text;
+			text = (string) note_text_cache [receipt.Uri];
+			// If text == null, this is equivalent to
+			// calling Delete (receipt.Uri)
+			TextCache.UserCache.WriteFromString (receipt.Uri, text);
+			note_text_cache.Remove (receipt.Uri);
+		}
+
 		override protected bool HitIsValid (Uri uri)
 		{
 			string note = Path.Combine (tomboy_dir, uri.Segments [1] + ".note");
@@ -205,5 +216,43 @@ namespace Beagle.Daemon.TomboyQueryable {
 
 			return false;
 		}
+
+		// IIndexableGenerator implementation
+		public string StatusName {
+			get { return "TomboyQueryable"; }
+		}
+		
+		private IEnumerator note_files = null;
+		
+		public void PostFlushHook () { }
+
+		public bool HasNextIndexable ()
+		{
+			if (note_files == null)
+				note_files = DirectoryWalker.GetFileInfos (tomboy_dir).GetEnumerator ();
+
+			return note_files.MoveNext ();
+		}
+
+		public Indexable GetNextIndexable ()
+		{
+			FileInfo file = (FileInfo) note_files.Current;
+
+			if (! file.Exists)
+				return null;
+
+			if (IsUpToDate (file.FullName))
+				return null;
+
+			Note note = TomboyNote.ParseNote (file);
+
+			if (note == null)
+				return null;
+
+			Indexable indexable = NoteToIndexable (file, note);
+			
+			return indexable;
+		}
+
 	}
 }
